


def get_parser(args):

    import argparse

    parser = argparse.ArgumentParser(description='Compute the Ambient Absorption imagery product for lidar')
    parser.add_argument('input',
                        help='PDAL-readable lidar content')
    parser.add_argument('--output',
                        help='Output filename', type=str, default='exposure')
    parser.add_argument('--filters',
                        help='Filter stages', type =str, default=None)
    parser.add_argument('--reader_args',
                        help='PDAL Reader args as JSON object', type =str, default=None)
    parser.add_argument('--resolution',
                        help='Raster output resolution', type =float, default=1.0)
    parser.add_argument('--alpha',
                        help='Amount of alpha for blend', type =float, default=0.85)
    parser.add_argument('--chunk_size',
                        help='PDAL streaming chunk size', type =int, default=int(1e6))
    parser.add_argument('--blue',
                        help='Use blue shade intensity', default=True)
    parser.add_argument('--debug',
                        action='store_true',
                        help='print debug messages to stderr')


    args = parser.parse_args(args)
    return args




import matplotlib as mpl
import matplotlib.pylab as plt
import uuid
__version__= '0.0.1'

import numpy as np
import sys
from osgeo import gdal
gdal.UseExceptions()


def readBand(filename,
             bandNumber,
             npDataType = np.float32,
             normalize = False):

    ds = gdal.Open(filename)
    band = ds.GetRasterBand(bandNumber)
    projection = ds.GetProjection()
    array = band.ReadAsArray().astype(npDataType)
    if normalize:
        array = (array - array.min()) / (array.max() - array.min())

    # mask off any nodata values
    array[array==band.GetNoDataValue()] = np.nan
    return array

def getInfo(filename):
    output = {}
    ds = gdal.Open(filename)
    projection = ds.GetProjection()
    output['projection'] = ds.GetProjection()
    output['metadata'] = ds.GetMetadata()
    return output
    ds.GetMetadata()
    return projection

class Blend(object):
    def __init__(self, data):
        self.data = data

    def do(self):
        intensityFilename = self.data.args.intensityPath
        daylightFilename = self.data.args.aoPath
        intensity = readBand(intensityFilename, 1, np.float32, True)
        daylight = readBand(daylightFilename, 1)

        if self.data.args.blue:
            intensity_RGBA = mpl.cm.Blues_r(intensity)
        else:
            intensity_RGBA = mpl.cm.Greys_r(intensity)

        intensity_RGBA[...,3] = np.full(intensity.shape, self.data.args.alpha)

        daylight_RGB = mpl.cm.Greys_r(daylight)

        RGBA = intensity_RGBA * 0.5 + daylight_RGB * 0.5

        numBands = RGBA.shape[2] 

        big = (RGBA*255).astype(np.uint8)

        tifpath = f"/vsimem/{str(uuid.uuid4())}.tif"
        gtif = gdal.GetDriverByName("GTiff")
        rast = gtif.Create(tifpath, intensity.shape[0], intensity.shape[1], numBands, gdal.GDT_Byte)

        info = getInfo(intensityFilename)

        rast.SetProjection(info['projection'])

        for b in range(numBands):
            rast.GetRasterBand(b+1).WriteArray(big[...,b])

        png = gdal.GetDriverByName("PNG")

        description='daylight exposure mixed with lidar intensity'
        title = 'Absorptive Daylight Exposure'
        png.CreateCopy( f"{self.data.args.output}.png", rast, 0,
            [ f'TITLE={title}', f'COMMENT={description}' ] )

        ds = gtif.CreateCopy( f"{self.data.args.output}.tif", rast, 0,
            [ 'COMPRESS=Deflate', 'TILED=YES','PREDICTOR=2' ] )


        if 'AREA_OR_POINT' in info['metadata']:
            ds.SetMetadataItem('AREA_OR_POINT', info['metadata']['AREA_OR_POINT'])

        ds.SetMetadataItem('TIFFTAG_SOFTWARE',f'Landrush exposure.py {__version__}')
        ds.SetMetadataItem('TIFFTAG_IMAGEDESCRIPTION',f'{description}')
        ds.SetMetadataItem('TIFFTAG_DOCUMENTNAME',f'{title}')

        ao = gdal.Open(self.data.args.aoPath)
        ds = gtif.CreateCopy( f"{self.data.args.output}-occlusion.tif", ao, 0,
            [ 'COMPRESS=Deflate', 'TILED=YES','PREDICTOR=2' ] )
 



from . import logs 
from . import do
from . import argparser
import sys


def main():
    args = argparser.get_parser(sys.argv[1:])
    if args.debug:
        logs.logger.setLevel(logs.logging.DEBUG)
        logs.handler.setLevel(logs.logging.DEBUG)

    do.doIt(args)


if __name__ == "__main__":
    sys.exit(main())





from . import logs

import json
import uuid
import tempfile
import subprocess
import shlex

import pdal


def run(command):
    args = shlex.split(command)
    p = subprocess.Popen(args,
                            stdin = subprocess.PIPE,
                            stdout = subprocess.PIPE,
                            stderr = subprocess.PIPE,)
    ret = p.communicate()

    if p.returncode != 0:
        error = ret[1].decode('utf-8','replace')
        raise RuntimeError(error)
    
    response = ret[0].decode('utf-8','replace')
    return response

class Data(object):
    def __init__(self, args):

        self.args = args
        self.checkValidData()
        self.args.intensityPath = tempfile.NamedTemporaryFile(suffix='.tif', delete=False).name
        self.args.dsmPath = tempfile.NamedTemporaryFile(suffix='.tif', delete=False).name
        self.args.aoPath = tempfile.NamedTemporaryFile(suffix='.tif', delete=False).name

    
    def checkValidData(self):
        reader = pdal.Reader(self.args.input)
        pipeline = reader.pipeline()
        qi = pipeline.quickinfo
        for key in qi:
            dimensions = [i.strip() for i in qi[key]['dimensions'].split(',')]
            if 'Intensity' not in dimensions:
                raise RuntimeError("Intensity information not available, this tool cannot run")

    def getReader(self):
        reader = None
        if self.args.reader_args:
            with open(self.args.reader_args,'r') as f:
                j = json.loads(f.read())
            reader = pdal.Reader(filename=self.args.input, *j)
        else:
            reader = pdal.Reader(filename=self.args.input)
        return reader

    def getWriters(self):
        intensity = pdal.Writer.gdal(
            filename=self.args.intensityPath,
            data_type='uint16_t',
            dimension='Intensity',
            output_type = 'idw',
            resolution=self.args.resolution,
        )
        dsm = pdal.Writer.gdal(
            filename=self.args.dsmPath,
            data_type='float',
            dimension='Z',
            output_type = 'idw',
            resolution=self.args.resolution,
        )
        return intensity | dsm 

    def getPipeline(self):
        reader = self.getReader()
        filters = self.getFilters()
        writers = self.getWriters()
        stage = None
        if filters:
            stage = reader | filters | writers
        else:
            stage = reader | writers

        return stage
        

    def execute(self):
        pipeline = self.getPipeline()
        count = 0
        if pipeline.streamable:
            count = pipeline.execute_streaming(chunk_size=self.args.chunk_size)
        else:
            count = pipeline.execute()
        logs.logger.info(f'Execute processed {count} points')
        logs.logger.info(f'Wrote itensity at {self.args.intensityPath}')
        logs.logger.info(f'Wrote dsm at {self.args.dsmPath}')

    def getFilters(self):
        if self.args.filters:
            with open(self.args.filters,'r') as f:
                j = json.loads(f.read())
                stages = pdal.pipeline._parse_stages(json.dumps(j))
                return pdal.Pipeline(stages)

        else:
            return None

    def getImageCenter(self):
        # Run our pipeline

        command = f'gdalinfo -json {self.args.dsmPath}'
        response = run(command)
        j = json.loads(response)
        corner = j['wgs84Extent']['coordinates'][0][0]
        logs.logger.info(f'Fetched image center {corner}')
        return corner
    def ambient_occlusion(self):
        lng, lat = self.getImageCenter()

        command = f"""whitebox_tools -r=TimeInDaylight  \
        -i {self.args.dsmPath} -o {self.args.aoPath}  --az_fraction=15.0 \
        --max_dist=100.0 --lat={lat:.5f} --long={lng:.5f} """
        logs.logger.info(f"Processed ambient occlusion '{command}'")
        response = run(command)




# Always prefer setuptools over distutils
from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='Trenchrun',  # Required
    version='0.0.1',  # Required
    description='Compute the Death Star trenchrun image from lidar data',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/hobuinc/trenchrun',
    author='Howard Butler',
    author_email='howard@hobu.co',
    classifiers=[  \
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.11',
    ],

    packages=find_packages(),  # Required
    python_requires='>=3.8',

    install_requires=['pdal','gdal','numpy','matplotlib'],

    entry_points={  # Optional
        'console_scripts': [
            'trenchrun=trenchrun.__main__:main',
        ],
    },

)






import logging
import sys

logger = logging.getLogger("absorption")
formatter = logging.Formatter("[%(levelname)s] %(message)s")
handler = logging.StreamHandler(stream=sys.stdout)
handler.setFormatter(formatter)
logger.addHandler(handler)

def setLevel(log_level):
    handler.setLevel(log_level)
    logger.setLevel(log_level)

